@@ -11,7 +11,7 @@
 #'data(antTraits)
 #'y <- as.matrix(antTraits$abund)
 #'# Fit gllvm model
-#'fit <- gllvm(y = y, family = "poisson")
+#'fit <- gllvm(y = y, family = poisson())
 #'# log-Likelihood:
 #'logLik(fit)
 #'
@@ -19,12 +19,17 @@
 
 logLik.gllvm <- function(object, ...)
 {
-  logL = object$logL
-  if(!is.null(object$params$inv.phi)){ object$params$inv.phi<-NULL; }
-  if(object$row.eff%in%c("fixed",TRUE)) object$params$row.params=object$params$row.params[-1]
-  if(object$row.eff=="random") object$params$row.params=NULL
-  attributes(logL)$df <- length(unlist(object$params))-object$num.lv*(object$num.lv-1)/2
+  logL <- object$logL
+  if (!is.null(object$params$inv.phi)) {
+    object$params$inv.phi <- NULL
+
+  }
+  if (object$row.eff %in% c("fixed", TRUE))
+    object$params$row.params <- object$params$row.params[-1]
+  if (object$row.eff == "random")
+    object$params$row.params <- NULL
+  attributes(logL)$df <- length(unlist(object$params)) - object$num.lv * (object$num.lv - 1) / 2
   attributes(logL)$nobs <- dim(object$y)[1]
   class(logL) <- "logLik"
-  return( logL )
+  return(logL)
 }
