@@ -11,7 +11,7 @@
 #' data(antTraits)
 #' y <- as.matrix(antTraits$abund)
 #'# Fit gllvm model
-#' fit <- gllvm(y = y, family = "poisson")
+#' fit <- gllvm(y = y, family = poisson())
 #' summary(fit)
 #'
 #'@export
@@ -28,7 +28,7 @@ summary.gllvm <- function(object, ...) {
 
   M <- cbind(object$params$beta0, object$params$theta)
 
-  newnams = c("Intercept")
+  newnams <- c("Intercept")
 
   if (num.lv > 0)
     newnams <- c(newnams, paste("theta.LV", 1:num.lv, sep = ""))
@@ -40,13 +40,13 @@ summary.gllvm <- function(object, ...) {
   print(object$family)
   cat("\n")
   cat("Coefficients: \n")
-
   print(M)
   cat("\n")
   if (!is.null(object$TR)) {
     if (!is.null(object$X)) {
       cat("Covariate coefficients: \n")
-      print(object$params$B)}
+      print(object$params$B)
+    }
     cat("\n")
   } else {
     if (!is.null(object$X)) {
@@ -61,22 +61,23 @@ summary.gllvm <- function(object, ...) {
     cat("\n")
   }
 
-  if (object$row.eff=="random") {
+  if (object$row.eff == "random") {
     cat("Variance of random row intercepts: \n")
-    object$params$sigma2=object$params$sigma^2;names(object$params$sigma2)="sigma^2"
+    object$params$sigma2 = object$params$sigma ^ 2
+    names(object$params$sigma2) = "sigma^2"
     print(object$params$sigma2)
     cat("\n")
   }
 
-  if(object$family=="negative.binomial"){
+  if (object$family == "negative.binomial") {
     cat("Dispersion parameters inv.phi: \n")
     print(object$params$inv.phi)
   }
-  if(object$family=="tweedie"){
+  if (object$family == "tweedie") {
     cat("Dispersion parameters: \n")
     print(object$params$phi)
   }
-  if(object$family=="ZIP"){
+  if (object$family == "ZIP") {
     cat("Zero inflation p: \n")
     print(object$params$phi)
   }
